/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
package com.ibm.rational.test.lm.spi.driver.test;

import java.net.URI;
import java.util.HashMap;

import com.ibm.ccl.soa.deploy.core.Topology;
import com.ibm.ccl.soa.deploy.operation.OperationUnit;
import com.ibm.rational.test.lm.spi.common.IRestAPIResponseCodes;
import com.ibm.rational.test.lm.spi.common.IRestResponse;
import com.ibm.rational.test.lm.spi.driver.test.util.ProviderXml;
import com.ibm.rational.test.lm.spi.driver.test.util.Registry;
import com.ibm.rational.test.lm.spi.driver.test.util.TestTracer;
import com.ibm.rational.test.lm.spi.driver.test.util.TopologyUtils;
import com.ibm.rational.test.lm.spi.driver.test.util.ProviderXml.ProviderServerInfo;

import junit.framework.TestCase;

public class Automation extends TestCase 
{
    private static String automationHostname = "";
    private static String automationPortNumber = "";
    
    private static final String SINGLE_SLASH = "/";
	private static final String COLON = ":";
    
    private static final String jobDefinitionUrlPart = "/jobDefinition/";
    private static final String jobExecutionUrlPart = "/jobExecution/";
    
    private static String baseJobDefinitionUrl;
    private static String baseJobExecutionUrl;
    
    private static HashMap<String, String> providerToConfig = new HashMap<String, String>();
    
    private static boolean isInited = false;
    
    protected void setUp() throws Exception
    {
        assertNotNull(Registry.getRegistry().getAutomationProvider());
        if (!isInited)
        {
            assertTrue(initSPIData());
            isInited = true;
        }
    }
    
    private boolean initSPIData()
    {
        boolean success = false;
        providerToConfig = ProviderXml.getProviderDocs();
        success = providerToConfig.size() > 0;
        ProviderServerInfo serverInfo = 
            ProviderXml.getProviderServerInfo(providerToConfig.get(Registry.getRegistry().getAutomationProviderPluginId()));
        automationHostname = serverInfo.hostname;
        automationPortNumber = serverInfo.port;
        baseJobDefinitionUrl = SINGLE_SLASH + automationHostname + COLON + automationPortNumber + jobDefinitionUrlPart;
        baseJobExecutionUrl = SINGLE_SLASH + automationHostname + COLON + automationPortNumber + jobExecutionUrlPart;        
        return success;
    }
    
    /**
     * test call of init() for automation provider
     */
    public void testInit()
    {
        try
        {
            Registry.getRegistry().getAutomationProvider().init(
                    providerToConfig.get(Registry.getRegistry().getAutomationProviderPluginId()),
                    new TestTracer());
        }
        catch (Exception e)
        {
            fail("Exception in Init(): " + e.toString());
        }
    }
    
    /**
     * test get() of /jobDefinition/ to retrieve ids of all jobs the provider knows about, in TDM Operational Model.
     */
    public void testGetJobDefinitionIds()
    {
        try
        {
            IRestResponse restResponse = Registry.getRegistry().getAutomationProvider().get(new URI(baseJobDefinitionUrl));
            assertTrue(restResponse.getResponseCode() == IRestAPIResponseCodes.SC_OK);
            String response = restResponse.getXMLResponse();
            assertNotNull(response);
            Topology jobDefs = TopologyUtils.parseTopologyFromXml(response);
            assertNotNull(jobDefs);
            assertTrue(jobDefs.getUnits().get(0) instanceof OperationUnit);
        }        
        catch (Exception e)
        {
            fail(e.toString());
        }        
    }

    /**
     * test get() of /jobDefinition/ to retrieve a TDM Model describing all known jobs.
     */
    public void testGetAllJobDefinitions()
    {
        try
        {
            IRestResponse restResponse = Registry.getRegistry().getAutomationProvider().get(new URI(baseJobDefinitionUrl + "*"));
            assertTrue(restResponse.getResponseCode() == IRestAPIResponseCodes.SC_OK);
            String response = restResponse.getXMLResponse();
            assertNotNull(response);
            Topology jobDefs = TopologyUtils.parseTopologyFromXml(response);
            assertNotNull(jobDefs);
            assertTrue(jobDefs.getUnits().get(0) instanceof OperationUnit);
        }        
        catch (Exception e)
        {
            fail(e.toString());
        }
    }
}
