/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
package com.ibm.rational.test.lm.spi.driver.test.util;

import java.io.File;
import java.io.IOException;
import java.io.StringReader;
import java.io.StringWriter;
import java.util.HashMap;
import java.util.List;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.jdom.output.XMLOutputter;

public class ProviderXml 
{
    private static final String CONFIG_FILE_LOCATION = "..\\com.ibm.rational.test.lm.spi\\samplexml\\integration_config.xml";
    
    private static final String ADAPTER_ELEMENT_NAME = "adapter";
    private static final String NAME_ELEMENT_NAME = "name";
    private static final String AUTOMATION_EXTENSION_ELEMENT_NAME = "automationExtensionId";
    private static final String INVENTORY_EXTENSION_ELEMENT_NAME = "inventoryExtensionId";
    private static final String VIRTUALIZATION_EXTENSION_ELEMENT_NAME = "virtualizationExtensionId";
    private static final String INSTANCE_ELEMENT_NAME = "instance";
    private static final String HOSTNAME_ELEMENT_NAME = "hostname";
    private static final String PORT_ELEMENT_NAME = "port";
    
    @SuppressWarnings("unchecked")
    public static HashMap<String, String> getProviderDocs()
    {
        SAXBuilder xmlIn = new SAXBuilder();
        HashMap<String, String> providerToConfig = new HashMap<String, String>();
        try
        {
            Document configDoc = xmlIn.build(new File(CONFIG_FILE_LOCATION));
            List<Element> children = 
                configDoc.getRootElement().getChildren(ADAPTER_ELEMENT_NAME);
            XMLOutputter xmlOut = new XMLOutputter();            
            for (Element child : children)
            {
                StringWriter stringWriter = new StringWriter();
                Document doclet = new Document();
                doclet.addContent((Element)child.clone());                
                xmlOut.output(doclet, stringWriter);
                if (child.getChild(INVENTORY_EXTENSION_ELEMENT_NAME) != null)
                {
                	String inventoryExtensionId = child.getChild(INVENTORY_EXTENSION_ELEMENT_NAME).getText();
                	if (inventoryExtensionId != null && inventoryExtensionId.length() > 0)
                	{
                		providerToConfig.put(inventoryExtensionId, stringWriter.toString());
                	}
                }
                if (child.getChild(AUTOMATION_EXTENSION_ELEMENT_NAME) != null)
                {
                    String automationExtensionId = child.getChild(AUTOMATION_EXTENSION_ELEMENT_NAME).getText();
                    if (automationExtensionId != null && automationExtensionId.length() > 0)
                    {
                        providerToConfig.put(automationExtensionId, stringWriter.toString());
                    }
                }
                if (child.getChild(VIRTUALIZATION_EXTENSION_ELEMENT_NAME) != null)
                {
                	String virtualizationExtensionId = child.getChild(VIRTUALIZATION_EXTENSION_ELEMENT_NAME).getText();
                	if (virtualizationExtensionId != null && virtualizationExtensionId.length() > 0)
                	{
                		providerToConfig.put(virtualizationExtensionId, stringWriter.toString());
                	}
                }
                stringWriter.flush();
                System.out.println("Loaded customization for provider: " + child.getChildText(NAME_ELEMENT_NAME));
            }            
        }
        catch (IOException e)
        {
            System.out.println("Problem loading provider customization file: " + e.toString());
        }
        catch (JDOMException e)
        {
            System.out.println("Problem parsing provider customization file: " + e.toString());
        }
        return providerToConfig;
    }
        
    public static class ProviderServerInfo
    {
        public String hostname;
        public String port;
    }
    
    @SuppressWarnings("unchecked")
    public static ProviderServerInfo getProviderServerInfo(String adapterConfig)
    {
        ProviderServerInfo serverInfo = new ProviderServerInfo();
        SAXBuilder xmlIn = new SAXBuilder();
        StringReader reader = new StringReader(adapterConfig);
        try
        {
            Document adapterDoc = xmlIn.build(reader);
            List<Element> instances = adapterDoc.getRootElement().getChildren(INSTANCE_ELEMENT_NAME);
            if (instances.size() == 0)
            {
                System.out.println("No instance data (<instance>) specified for adapter");
            }
            if (instances.size() > 1)
            {
                System.out.println("Currently only ONE instance of an adapter is supported");
            }
            serverInfo.hostname = instances.get(0).getChild(HOSTNAME_ELEMENT_NAME).getText();		
            serverInfo.port = instances.get(0).getChild(PORT_ELEMENT_NAME).getText();
        }
        catch (IOException e)
        {
            //can't happen
        }
        catch (JDOMException e)
        {
            System.out.println("Problem parsing adapter portion of customization file: " + e.toString());
        }
        return serverInfo;
    }
}
