/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
package com.ibm.rational.test.lm.spi.sample.automation;

import java.net.URI;

import com.ibm.ccl.soa.deploy.core.CoreFactory;
import com.ibm.ccl.soa.deploy.core.Topology;
import com.ibm.ccl.soa.deploy.operation.OperationFactory;
import com.ibm.ccl.soa.deploy.operation.OperationUnit;
import com.ibm.rational.test.lm.spi.IAutomationSPI;
import com.ibm.rational.test.lm.spi.ITracer;
import com.ibm.rational.test.lm.spi.common.IRestAPIResponseCodes;
import com.ibm.rational.test.lm.spi.common.IRestResponse;
import com.ibm.rational.test.lm.spi.common.SPIURI;
import com.ibm.rational.test.lm.spi.common.URIResourceType;
import com.ibm.rational.test.lm.spi.sample.util.AdapterXml;
import com.ibm.rational.test.lm.spi.sample.util.RestResponse;
import com.ibm.rational.test.lm.spi.sample.util.TopologyUtils;

/**
 * Implementation of the com.ibm.rational.test.lm.spi.automation SPI extension point
 */
public class AutomationSPI implements IAutomationSPI 
{
    public AutomationSPI() 
    {

    }

    public IRestResponse delete(URI uri) 
    {
        RestResponse response = new RestResponse();
        response.setResponseCode(IRestAPIResponseCodes.SC_NOT_IMPLEMENTED);
        return response;
    }

    public IRestResponse get(URI uri) 
    {
        RestResponse response = new RestResponse();                    
        SPIURI spiuri = SPIURI.parse(uri);
        URIResourceType resourceType = spiuri.getResourceType();
        if (resourceType.equals(URIResourceType.jobDefinition))
        {
            try
            {
                String id = spiuri.getResourceID();
                if (id == null || id.equals(""))
                {
                    response.setXMLResponse(getJobIds());
                }
                else if (id.equals("*"))
                {
                    response.setXMLResponse(getAllJobs());
                }
                else
                {
                    response.setXMLResponse(getJob(id));
                }
                response.setResponseCode(IRestAPIResponseCodes.SC_OK);
            }
            catch (Exception e)
            {
                response.setResponseCode(IRestAPIResponseCodes.SC_INTERNAL_SERVER_ERROR);
            }
        }
        else
        {
            response.setResponseCode(IRestAPIResponseCodes.SC_NOT_IMPLEMENTED);
        }
        return response;
    }

    public void init(String initProperties, ITracer tracer) throws Exception 
    {
        System.out.println("Initing Automation Sample with hostname: " + AdapterXml.getHostname(initProperties));
        System.out.println("Initing Automation Sample with port: " + AdapterXml.getPort(initProperties));
    }

    public IRestResponse post(URI uri, String XMLBody) 
    {
        RestResponse response = new RestResponse();
        response.setResponseCode(IRestAPIResponseCodes.SC_NOT_IMPLEMENTED);
        return response;
    }

    private String getJob(String id)
    {
        //TODO: properly implement multiple jobs
        return getAllJobs();
    }
    
    private String getAllJobs()
    {
        Topology jobTopology = CoreFactory.eINSTANCE.createTopology();
        OperationUnit operationUnit = OperationFactory.eINSTANCE.createOperationUnit("Automation", null, new String[] {}, null);
        operationUnit.getOperation().setOperationId("1");
        operationUnit.getOperation().setOperationName("Sample Job 1");
        operationUnit.getOperation().setDisplayName("Sample Job 1");
        operationUnit.getOperation().setDescription("This is a sample Job");
        jobTopology.getUnits().add(operationUnit);
        return TopologyUtils.topologyToXml(jobTopology); 
    }
    
    private String getJobIds()
    {
        Topology jobTopology = CoreFactory.eINSTANCE.createTopology();
        OperationUnit operationUnit = OperationFactory.eINSTANCE.createOperationUnit("Automation", null, new String[] {}, null);
        operationUnit.getOperation().setOperationId("1");
        jobTopology.getUnits().add(operationUnit);
        return TopologyUtils.topologyToXml(jobTopology);
    }
}
